﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Syncfusion.Windows.Controls.Gantt;
using Syncfusion.Windows.Controls.Gantt.Chart;
using System.Windows.Controls.Primitives;
using System.Windows;

namespace ResourceView
{
    public class PreventDragAndDropHelper
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PreventDragAndDropHelper"/> class.
        /// </summary>
        /// <param name="Gantt">The gantt.</param>
        public PreventDragAndDropHelper(GanttControl Gantt)
        {
            ganttControl = Gantt;
            ganttControl.Loaded += new RoutedEventHandler(ganttControl_Loaded);
        }

        #region Private Properties

        private GanttControl ganttControl;
        bool cancelDrop = false;
        DateTime OldStartTime;
        DateTime OldEndTime;

        #endregion

        /// <summary>
        /// Handles the Loaded event of the ganttControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        void ganttControl_Loaded(object sender, RoutedEventArgs e)
        {
            ganttControl.GanttGrid.ReadOnly = true;

            // Customizing the header text of the Grid.
            ganttControl.GanttGrid.Columns[0].HeaderText = "Resource Name";
            ganttControl.GanttGrid.Columns[1].HeaderText = "Start Date";
            ganttControl.GanttGrid.Columns[2].HeaderText = "Finish Date";

            //Handling thumbs event to prevent the overlapping while drag and drop or resizing the TaskNode
            HandlingNodeThumbsEvents(ganttControl);
        }

        /// <summary>
        /// Handlings the node thumbs events.
        /// </summary>
        /// <param name="gantt">The gantt.</param>
        private void HandlingNodeThumbsEvents(GanttControl gantt)
        {
            //Iterating the GanttChartItemPanel of Gantt control
            GanttChartItemsPanel ganttChartItemsPanel = gantt.FindElementOfType<GanttChartItemsPanel>(); 
            for (int i = 0; i < ganttChartItemsPanel.Children.Count; i++)
            {
                GanttChartRow ganttChartRow = ganttChartItemsPanel.Children[i] as GanttChartRow;

                if (ganttChartRow != null)
                {
                    //Iterating the GanttChartRowItemsPresenter of every GanttChartRow.
                    GanttChartRowItemsPresenter ganttChartRowItemsPresenter = ganttChartRow.FindElementOfType<GanttChartRowItemsPresenter>(); 
                    for (int j = 0; j < ganttChartRowItemsPresenter.Children.Count; j++)
                    {
                        //iterating the GanttNode from GanttChartRowItemsPresenter to hook the events for Thumbs.
                        GanttNode ganttNode = ganttChartRowItemsPresenter.Children[j] as GanttNode;
                        var leftThumb = (Thumb)ganttNode.Template.FindName("PART_LeftThumb", ganttNode);
                        if (leftThumb != null)
                        {
                            leftThumb.DragDelta += this.nodeThumb_DragDelta;
                            leftThumb.DragCompleted += this.nodeThump_DragCompleted;
                        }

                        var rightThumb = (Thumb)ganttNode.Template.FindName("PART_RightThumb", ganttNode);
                        if (rightThumb != null)
                        {
                            rightThumb.DragDelta += this.nodeThumb_DragDelta;
                            rightThumb.DragCompleted += nodeThump_DragCompleted;
                        }
                        var dragThump = (Thumb)ganttNode.Template.FindName("PART_DragDropThumb", ganttNode);
                        if (dragThump != null)
                        {
                            dragThump.DragDelta += nodeThumb_DragDelta;
                            dragThump.DragCompleted += nodeThump_DragCompleted;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Handles the DragCompleted event of the nodeThump control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Controls.Primitives.DragCompletedEventArgs"/> instance containing the event data.</param>
        void nodeThump_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            Thumb t = sender as Thumb;
            GanttNode node = t.TemplatedParent as GanttNode;
            GanttChartRowItemsPresenter itemsPresenter = node.Parent as GanttChartRowItemsPresenter;

            List<GanttNode> nodeCollection = new List<GanttNode>();
            
            //Getting the collection of GanttNodes in same row.
            foreach (UIElement child in itemsPresenter.Children)
            {
                if (child is GanttNode)
                {
                    GanttNode ganttNode = child as GanttNode;
                    nodeCollection.Add(ganttNode);
                }
            }

            //removing the current node from the collection.
            nodeCollection.Remove(node);

            for (int i = 0; i < nodeCollection.Count; i++)
            {
                //Checking whether the node is overlapping the another node while Drag and drop or resizing the node.
                if (nodeCollection[i].StartTime.CompareTo(node.StartTime) >= 0 && nodeCollection[i].EndTime.CompareTo(node.EndTime) <= 0)
                {
                    cancelDrop = true;
                }
                else if (node.EndTime.CompareTo(nodeCollection[i].StartTime) >= 0 && node.EndTime.CompareTo(nodeCollection[i].EndTime) <= 0)
                {
                    cancelDrop = true;
                }
                else if (node.StartTime.CompareTo(nodeCollection[i].StartTime) >= 0 && node.StartTime.CompareTo(nodeCollection[i].EndTime) <= 0)
                {
                    cancelDrop = true;
                }
            }

            //When the node overlaps the another node cancelDrop will be "true".
            if (cancelDrop)
            {
                //if cancelDrop is true the node will be retained to old position.
                node.StartTime = OldStartTime;
                node.EndTime = OldEndTime;
                cancelDrop = false;
            }
        }

        /// <summary>
        /// Handles the DragDelta event of the nodeThumb control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Controls.Primitives.DragDeltaEventArgs"/> instance containing the event data.</param>
        private void nodeThumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            Thumb thumb = sender as Thumb;
            GanttNode node = thumb.TemplatedParent as GanttNode;

            //Storing the old value's in variables these are all used when the Task Nodes are overlapped.
            OldStartTime = node.StartTime;
            OldEndTime = node.EndTime;
        }
    }
}
